@@ -5,8 +5,12 @@
 */
 
 #include "perf.h"
+
+#include <botan/assert.h>
 #include <set>
 
+#include <botan/symkey.h>
+
 #if defined(BOTAN_HAS_BLOCK_CIPHER)
    #include <botan/block_cipher.h>
 #endif
@@ -409,6 +413,7 @@ std::unique_ptr<PerfTest> PerfTest::get_sym(const std::string& alg) {
    }
 #endif
 
+   BOTAN_UNUSED(alg);
    return {};
 }
 
