@@ -10,6 +10,8 @@
 #include "cli.h"
 #include "sandbox.h"
 
+#include <botan/internal/target_info.h>
+
 #if defined(BOTAN_TARGET_OS_HAS_SOCKETS)
    #include <sys/socket.h>
 #endif
@@ -32,8 +34,8 @@
    #include <botan/tls_policy.h>
    #include <botan/tls_server.h>
    #include <botan/tls_session_manager_memory.h>
-   #include <botan/internal/os_utils.h>
 
+   #include <chrono>
    #include <fstream>
    #include <list>
    #include <memory>
@@ -169,8 +171,6 @@ class TLS_Server final : public Command {
             std::make_shared<Basic_Credentials_Manager>(server_cred, server_key, std::move(psk), psk_identity, psk_prf);
          auto callbacks = std::make_shared<Callbacks>(*this);
 
-         output() << "Listening for new connections on " << transport << " port " << port << std::endl;
-
          if(!m_sandbox.init()) {
             error_output() << "Failed sandboxing\n";
             return;
@@ -179,6 +179,8 @@ class TLS_Server final : public Command {
          socket_type server_fd = make_server_socket(port);
          size_t clients_served = 0;
 
+         output() << "Listening for new connections on " << transport << " port " << port << std::endl;
+
          while(true) {
             if(max_clients > 0 && clients_served >= max_clients) {
                break;
@@ -223,7 +225,8 @@ class TLS_Server final : public Command {
             std::unique_ptr<std::ostream> dump_stream;
 
             if(!dump_traces_to.empty()) {
-               uint64_t timestamp = Botan::OS::get_high_resolution_clock();
+               auto now = std::chrono::system_clock::now().time_since_epoch();
+               uint64_t timestamp = std::chrono::duration_cast<std::chrono::nanoseconds>(now).count();
                const std::string dump_file = dump_traces_to + "/tls_" + std::to_string(timestamp) + ".bin";
                dump_stream = std::make_unique<std::ofstream>(dump_file.c_str());
             }
