@@ -8,17 +8,24 @@
 
 #include <botan/internal/bit_ops.h>
 #include <botan/internal/bswap.h>
-#include <botan/internal/cpuid.h>
 #include <botan/internal/ct_utils.h>
 #include <botan/internal/loadstor.h>
 #include <botan/internal/rotate.h>
 
-namespace Botan {
+#if defined(BOTAN_HAS_CPUID)
+   #include <botan/internal/cpuid.h>
+#endif
 
 #if defined(BOTAN_HAS_AES_POWER8) || defined(BOTAN_HAS_AES_ARMV8) || defined(BOTAN_HAS_AES_NI)
    #define BOTAN_HAS_HW_AES_SUPPORT
 #endif
 
+#if defined(BOTAN_HAS_HW_AES_SUPPORT)
+   #include <bit>
+#endif
+
+namespace Botan {
+
 /*
 * One of three AES implementation strategies are used to get a constant time
 * implementation which is immune to common cache/timing based side channels:
@@ -408,43 +415,43 @@ inline void ks_expand(uint32_t B[8], const uint32_t K[], size_t r) {
 
 inline void shift_rows(uint32_t B[8]) {
    // 3 0 1 2 7 4 5 6 10 11 8 9 14 15 12 13 17 18 19 16 21 22 23 20 24 25 26 27 28 29 30 31
-#if defined(BOTAN_TARGET_CPU_HAS_NATIVE_64BIT)
-   for(size_t i = 0; i != 8; i += 2) {
-      uint64_t x = (static_cast<uint64_t>(B[i]) << 32) | B[i + 1];
-      x = bit_permute_step<uint64_t>(x, 0x0022331100223311, 2);
-      x = bit_permute_step<uint64_t>(x, 0x0055005500550055, 1);
-      B[i] = static_cast<uint32_t>(x >> 32);
-      B[i + 1] = static_cast<uint32_t>(x);
+   if constexpr(HasNative64BitRegisters) {
+      for(size_t i = 0; i != 8; i += 2) {
+         uint64_t x = (static_cast<uint64_t>(B[i]) << 32) | B[i + 1];
+         x = bit_permute_step<uint64_t>(x, 0x0022331100223311, 2);
+         x = bit_permute_step<uint64_t>(x, 0x0055005500550055, 1);
+         B[i] = static_cast<uint32_t>(x >> 32);
+         B[i + 1] = static_cast<uint32_t>(x);
+      }
+   } else {
+      for(size_t i = 0; i != 8; ++i) {
+         uint32_t x = B[i];
+         x = bit_permute_step<uint32_t>(x, 0x00223311, 2);
+         x = bit_permute_step<uint32_t>(x, 0x00550055, 1);
+         B[i] = x;
+      }
    }
-#else
-   for(size_t i = 0; i != 8; ++i) {
-      uint32_t x = B[i];
-      x = bit_permute_step<uint32_t>(x, 0x00223311, 2);
-      x = bit_permute_step<uint32_t>(x, 0x00550055, 1);
-      B[i] = x;
-   }
-#endif
 }
 
 inline void inv_shift_rows(uint32_t B[8]) {
    // Inverse of shift_rows, just inverting the steps
 
-#if defined(BOTAN_TARGET_CPU_HAS_NATIVE_64BIT)
-   for(size_t i = 0; i != 8; i += 2) {
-      uint64_t x = (static_cast<uint64_t>(B[i]) << 32) | B[i + 1];
-      x = bit_permute_step<uint64_t>(x, 0x0055005500550055, 1);
-      x = bit_permute_step<uint64_t>(x, 0x0022331100223311, 2);
-      B[i] = static_cast<uint32_t>(x >> 32);
-      B[i + 1] = static_cast<uint32_t>(x);
+   if constexpr(HasNative64BitRegisters) {
+      for(size_t i = 0; i != 8; i += 2) {
+         uint64_t x = (static_cast<uint64_t>(B[i]) << 32) | B[i + 1];
+         x = bit_permute_step<uint64_t>(x, 0x0055005500550055, 1);
+         x = bit_permute_step<uint64_t>(x, 0x0022331100223311, 2);
+         B[i] = static_cast<uint32_t>(x >> 32);
+         B[i + 1] = static_cast<uint32_t>(x);
+      }
+   } else {
+      for(size_t i = 0; i != 8; ++i) {
+         uint32_t x = B[i];
+         x = bit_permute_step<uint32_t>(x, 0x00550055, 1);
+         x = bit_permute_step<uint32_t>(x, 0x00223311, 2);
+         B[i] = x;
+      }
    }
-#else
-   for(size_t i = 0; i != 8; ++i) {
-      uint32_t x = B[i];
-      x = bit_permute_step<uint32_t>(x, 0x00550055, 1);
-      x = bit_permute_step<uint32_t>(x, 0x00223311, 2);
-      B[i] = x;
-   }
-#endif
 }
 
 inline void mix_columns(uint32_t B[8]) {
@@ -740,14 +747,20 @@ void aes_key_schedule(const uint8_t key[],
 }
 
 size_t aes_parallelism() {
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return 8;  // pipelined
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return 4;  // pipelined
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return 2;  // pipelined
    }
 #endif
@@ -756,16 +769,22 @@ size_t aes_parallelism() {
    return 2;
 }
 
-const char* aes_provider() {
+std::string aes_provider() {
+#if defined(BOTAN_HAS_AES_VAES)
+   if(auto feat = CPUID::check(CPUID::Feature::AVX2_AES)) {
+      return *feat;
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
-      return "cpu";
+   if(auto feat = CPUID::check(CPUID::Feature::HW_AES)) {
+      return *feat;
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
-      return "vperm";
+   if(auto feat = CPUID::check(CPUID::Feature::SIMD_4X32)) {
+      return *feat;
    }
 #endif
 
@@ -813,14 +832,20 @@ bool AES_256::has_keying_material() const {
 void AES_128::encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    assert_key_material_set();
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return x86_vaes_encrypt_n(in, out, blocks);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return hw_aes_encrypt_n(in, out, blocks);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_encrypt_n(in, out, blocks);
    }
 #endif
@@ -831,14 +856,20 @@ void AES_128::encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const
 void AES_128::decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    assert_key_material_set();
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return x86_vaes_decrypt_n(in, out, blocks);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return hw_aes_decrypt_n(in, out, blocks);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_decrypt_n(in, out, blocks);
    }
 #endif
@@ -848,19 +879,26 @@ void AES_128::decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const
 
 void AES_128::key_schedule(std::span<const uint8_t> key) {
 #if defined(BOTAN_HAS_AES_NI)
-   if(CPUID::has_aes_ni()) {
+   if(CPUID::has(CPUID::Feature::AESNI)) {
       return aesni_key_schedule(key.data(), key.size());
    }
 #endif
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, true);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
-      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, CPUID::is_little_endian());
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
+      constexpr bool is_little_endian = std::endian::native == std::endian::little;
+      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, is_little_endian);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_key_schedule(key.data(), key.size());
    }
 #endif
@@ -876,14 +914,20 @@ void AES_128::clear() {
 void AES_192::encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    assert_key_material_set();
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return x86_vaes_encrypt_n(in, out, blocks);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return hw_aes_encrypt_n(in, out, blocks);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_encrypt_n(in, out, blocks);
    }
 #endif
@@ -894,14 +938,20 @@ void AES_192::encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const
 void AES_192::decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    assert_key_material_set();
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return x86_vaes_decrypt_n(in, out, blocks);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return hw_aes_decrypt_n(in, out, blocks);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_decrypt_n(in, out, blocks);
    }
 #endif
@@ -911,19 +961,26 @@ void AES_192::decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const
 
 void AES_192::key_schedule(std::span<const uint8_t> key) {
 #if defined(BOTAN_HAS_AES_NI)
-   if(CPUID::has_aes_ni()) {
+   if(CPUID::has(CPUID::Feature::AESNI)) {
       return aesni_key_schedule(key.data(), key.size());
    }
 #endif
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, true);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
-      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, CPUID::is_little_endian());
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
+      constexpr bool is_little_endian = std::endian::native == std::endian::little;
+      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, is_little_endian);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_key_schedule(key.data(), key.size());
    }
 #endif
@@ -939,14 +996,20 @@ void AES_192::clear() {
 void AES_256::encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    assert_key_material_set();
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return x86_vaes_encrypt_n(in, out, blocks);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return hw_aes_encrypt_n(in, out, blocks);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_encrypt_n(in, out, blocks);
    }
 #endif
@@ -957,14 +1020,20 @@ void AES_256::encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const
 void AES_256::decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    assert_key_material_set();
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return x86_vaes_decrypt_n(in, out, blocks);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
       return hw_aes_decrypt_n(in, out, blocks);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_decrypt_n(in, out, blocks);
    }
 #endif
@@ -974,19 +1043,26 @@ void AES_256::decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const
 
 void AES_256::key_schedule(std::span<const uint8_t> key) {
 #if defined(BOTAN_HAS_AES_NI)
-   if(CPUID::has_aes_ni()) {
+   if(CPUID::has(CPUID::Feature::AESNI)) {
       return aesni_key_schedule(key.data(), key.size());
    }
 #endif
 
+#if defined(BOTAN_HAS_AES_VAES)
+   if(CPUID::has(CPUID::Feature::AVX2_AES)) {
+      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, true);
+   }
+#endif
+
 #if defined(BOTAN_HAS_HW_AES_SUPPORT)
-   if(CPUID::has_hw_aes()) {
-      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, CPUID::is_little_endian());
+   if(CPUID::has(CPUID::Feature::HW_AES)) {
+      constexpr bool is_little_endian = std::endian::native == std::endian::little;
+      return aes_key_schedule(key.data(), key.size(), m_EK, m_DK, is_little_endian);
    }
 #endif
 
 #if defined(BOTAN_HAS_AES_VPERM)
-   if(CPUID::has_vperm()) {
+   if(CPUID::has(CPUID::Feature::SIMD_4X32)) {
       return vperm_key_schedule(key.data(), key.size());
    }
 #endif
