@@ -10,6 +10,7 @@
 #define BOTAN_MODE_PADDING_H_
 
 #include <botan/secmem.h>
+#include <memory>
 #include <string>
 
 namespace Botan {
@@ -67,7 +68,7 @@ class BOTAN_TEST_API BlockCipherModePaddingMethod {
 /**
 * PKCS#7 Padding
 */
-class BOTAN_TEST_API PKCS7_Padding final : public BlockCipherModePaddingMethod {
+class BOTAN_FUZZER_API PKCS7_Padding final : public BlockCipherModePaddingMethod {
    public:
       void add_padding(secure_vector<uint8_t>& buffer, size_t final_block_bytes, size_t block_size) const override;
 
@@ -81,7 +82,7 @@ class BOTAN_TEST_API PKCS7_Padding final : public BlockCipherModePaddingMethod {
 /**
 * ANSI X9.23 Padding
 */
-class BOTAN_TEST_API ANSI_X923_Padding final : public BlockCipherModePaddingMethod {
+class BOTAN_FUZZER_API ANSI_X923_Padding final : public BlockCipherModePaddingMethod {
    public:
       void add_padding(secure_vector<uint8_t>& buffer, size_t final_block_bytes, size_t block_size) const override;
 
@@ -95,7 +96,7 @@ class BOTAN_TEST_API ANSI_X923_Padding final : public BlockCipherModePaddingMeth
 /**
 * One And Zeros Padding (ISO/IEC 9797-1, padding method 2)
 */
-class BOTAN_TEST_API OneAndZeros_Padding final : public BlockCipherModePaddingMethod {
+class BOTAN_FUZZER_API OneAndZeros_Padding final : public BlockCipherModePaddingMethod {
    public:
       void add_padding(secure_vector<uint8_t>& buffer, size_t final_block_bytes, size_t block_size) const override;
 
@@ -109,7 +110,7 @@ class BOTAN_TEST_API OneAndZeros_Padding final : public BlockCipherModePaddingMe
 /**
 * ESP Padding (RFC 4303)
 */
-class BOTAN_TEST_API ESP_Padding final : public BlockCipherModePaddingMethod {
+class BOTAN_FUZZER_API ESP_Padding final : public BlockCipherModePaddingMethod {
    public:
       void add_padding(secure_vector<uint8_t>& buffer, size_t final_block_bytes, size_t block_size) const override;
 
