@@ -9,12 +9,12 @@
 #include <botan/tpm2_rsa.h>
 
 #include <botan/hash.h>
+#include <botan/pss_params.h>
 #include <botan/rsa.h>
 
 #include <botan/internal/ct_utils.h>
 #include <botan/internal/emsa.h>
 #include <botan/internal/fmt.h>
-#include <botan/internal/pss_params.h>
 #include <botan/internal/scan_name.h>
 #include <botan/internal/stl_util.h>
 #include <botan/internal/tpm2_algo_mappings.h>
@@ -27,15 +27,32 @@
 namespace Botan::TPM2 {
 
 RSA_PublicKey::RSA_PublicKey(Object handle, SessionBundle session_bundle, const TPM2B_PUBLIC* public_blob) :
+      Botan::TPM2::RSA_PublicKey(
+         std::move(handle), std::move(session_bundle), rsa_pubkey_components_from_tss2_public(public_blob)) {}
+
+RSA_PublicKey::RSA_PublicKey(Object handle, SessionBundle session_bundle, const std::pair<BigInt, BigInt>& pubkey) :
       Botan::TPM2::PublicKey(std::move(handle), std::move(session_bundle)),
-      Botan::RSA_PublicKey(rsa_pubkey_from_tss2_public(public_blob)) {}
+
+      // TODO: move those BigInts as soon as the RSA c'tor allows it
+      Botan::RSA_PublicKey(pubkey.first, pubkey.second) {}
 
 RSA_PrivateKey::RSA_PrivateKey(Object handle,
                                SessionBundle session_bundle,
                                const TPM2B_PUBLIC* public_blob,
                                std::span<const uint8_t> private_blob) :
+      Botan::TPM2::RSA_PrivateKey(std::move(handle),
+                                  std::move(session_bundle),
+                                  rsa_pubkey_components_from_tss2_public(public_blob),
+                                  private_blob) {}
+
+RSA_PrivateKey::RSA_PrivateKey(Object handle,
+                               SessionBundle session_bundle,
+                               const std::pair<BigInt, BigInt>& pubkey,
+                               std::span<const uint8_t> private_blob) :
       Botan::TPM2::PrivateKey(std::move(handle), std::move(session_bundle), private_blob),
-      Botan::RSA_PublicKey(rsa_pubkey_from_tss2_public(public_blob)) {}
+
+      // TODO: move those BigInts as soon as the RSA c'tor allows it
+      Botan::RSA_PublicKey(pubkey.first, pubkey.second) {}
 
 std::unique_ptr<TPM2::PrivateKey> RSA_PrivateKey::create_unrestricted_transient(const std::shared_ptr<Context>& ctx,
                                                                                 const SessionBundle& sessions,
@@ -173,12 +190,12 @@ class RSA_Signature_Operation final : public Signature_Operation {
             return AlgorithmIdentifier(oid, AlgorithmIdentifier::USE_EMPTY_PARAM);
          } catch(Lookup_Error&) {}
 
-         if(emsa_name.starts_with("EMSA4(")) {
+         if(emsa_name.starts_with("PSS(")) {
             auto parameters = PSS_Params::from_emsa_name(emsa_name).serialize();
-            return AlgorithmIdentifier("RSA/EMSA4", parameters);
+            return AlgorithmIdentifier("RSA/PSS", parameters);
          }
 
-         throw Not_Implemented("No algorithm identifier defined for RSA with " + emsa_name);
+         throw Invalid_Argument(fmt("Signatures using RSA/{} are not supported", emsa_name));
       }
 
    private:
@@ -340,13 +357,25 @@ class RSA_Decryption_Operation final : public PK_Ops::Decryption {
                                                                         &label,
                                                                         out_ptr(plaintext)));
 
-         valid_mask = CT::Mask<uint8_t>::is_equal(rc, TPM2_RC_SUCCESS).value();
-         if(rc == TPM2_RC_SUCCESS) {
-            BOTAN_ASSERT_NONNULL(plaintext);
-            return copy_into<secure_vector<uint8_t>>(*plaintext);
-         } else {
-            return {};
-         }
+         const auto success = CT::Mask<decltype(rc)>::is_equal(rc, TPM2_RC_SUCCESS).as_choice();
+         valid_mask = CT::Mask<uint8_t>::from_choice(success).value();
+
+         // A "typical" payload size for RSA encryption, assuming that we usually
+         // encrypt some symmetric key of a hybrid encryption scheme.
+         constexpr size_t default_plaintext_length = 32;
+
+         // When Esys_RSA_Decrypt fails to decrypt the ciphertext (e.g. because
+         // of a PKCS#1.5 padding failure), the `plaintext` pointer will be nullptr.
+         // This behaviour in itself is likely exposing a timing side channel already.
+         // Nevertheless, we do our best to mitigate any oracles by always copying a
+         // dummy plaintext value in this case.
+         auto dummy_plaintext = init_with_size<TPM2B_PUBLIC_KEY_RSA>(default_plaintext_length);
+         auto* out = &dummy_plaintext;
+         auto* maybe_plaintext = plaintext.get();
+         CT::conditional_swap_ptr(success.as_bool(), out, maybe_plaintext);
+
+         BOTAN_ASSERT_NONNULL(out);
+         return copy_into<secure_vector<uint8_t>>(*out);
       }
 
       size_t plaintext_length(size_t /* ciphertext_length */) const override {
