@@ -9,17 +9,20 @@
 
 #include <botan/ber_dec.h>
 #include <botan/der_enc.h>
-#include <botan/reducer.h>
+#include <botan/numthry.h>
+#include <botan/pss_params.h>
+#include <botan/internal/barrett.h>
 #include <botan/internal/blinding.h>
 #include <botan/internal/divide.h>
 #include <botan/internal/emsa.h>
 #include <botan/internal/fmt.h>
 #include <botan/internal/keypair.h>
+#include <botan/internal/mod_inv.h>
 #include <botan/internal/monty.h>
 #include <botan/internal/monty_exp.h>
 #include <botan/internal/parsing.h>
 #include <botan/internal/pk_ops_impl.h>
-#include <botan/internal/pss_params.h>
+#include <botan/internal/target_info.h>
 #include <botan/internal/workfactor.h>
 
 #if defined(BOTAN_HAS_THREAD_UTILS)
@@ -33,14 +36,15 @@ class RSA_Public_Data final {
       RSA_Public_Data(BigInt&& n, BigInt&& e) :
             m_n(std::move(n)),
             m_e(std::move(e)),
-            m_monty_n(std::make_shared<Montgomery_Params>(m_n)),
+            m_mod_n(Barrett_Reduction::for_public_modulus(m_n)),
+            m_monty_n(std::make_shared<Montgomery_Params>(m_n, m_mod_n)),
             m_public_modulus_bits(m_n.bits()),
             m_public_modulus_bytes(m_n.bytes()) {}
 
       BigInt public_op(const BigInt& m) const {
          const size_t powm_window = 1;
          auto powm_m_n = monty_precompute(m_monty_n, m, powm_window, false);
-         return monty_execute_vartime(*powm_m_n, m_e);
+         return monty_execute_vartime(*powm_m_n, m_e).value();
       }
 
       const BigInt& get_n() const { return m_n; }
@@ -51,9 +55,14 @@ class RSA_Public_Data final {
 
       size_t public_modulus_bytes() const { return m_public_modulus_bytes; }
 
+      const std::shared_ptr<const Montgomery_Params>& monty_n() const { return m_monty_n; }
+
+      const Barrett_Reduction& reducer_mod_n() const { return m_mod_n; }
+
    private:
       BigInt m_n;
       BigInt m_e;
+      Barrett_Reduction m_mod_n;
       std::shared_ptr<const Montgomery_Params> m_monty_n;
       size_t m_public_modulus_bits;
       size_t m_public_modulus_bytes;
@@ -68,10 +77,9 @@ class RSA_Private_Data final {
             m_d1(std::move(d1)),
             m_d2(std::move(d2)),
             m_c(std::move(c)),
-            m_mod_p(m_p),
-            m_mod_q(m_q),
-            m_monty_p(std::make_shared<Montgomery_Params>(m_p, m_mod_p)),
-            m_monty_q(std::make_shared<Montgomery_Params>(m_q, m_mod_q)),
+            m_monty_p(std::make_shared<Montgomery_Params>(m_p)),
+            m_monty_q(std::make_shared<Montgomery_Params>(m_q)),
+            m_c_monty(m_monty_p, m_c),
             m_p_bits(m_p.bits()),
             m_q_bits(m_q.bits()) {}
 
@@ -87,9 +95,7 @@ class RSA_Private_Data final {
 
       const BigInt& get_c() const { return m_c; }
 
-      const Modular_Reducer& mod_p() const { return m_mod_p; }
-
-      const Modular_Reducer& mod_q() const { return m_mod_q; }
+      const Montgomery_Int& get_c_monty() const { return m_c_monty; }
 
       const std::shared_ptr<const Montgomery_Params>& monty_p() const { return m_monty_p; }
 
@@ -99,6 +105,8 @@ class RSA_Private_Data final {
 
       size_t q_bits() const { return m_q_bits; }
 
+      bool primes_imbalanced() const { return p_bits() != q_bits(); }
+
    private:
       BigInt m_d;
       BigInt m_p;
@@ -107,10 +115,9 @@ class RSA_Private_Data final {
       BigInt m_d2;
       BigInt m_c;
 
-      Modular_Reducer m_mod_p;
-      Modular_Reducer m_mod_q;
       std::shared_ptr<const Montgomery_Params> m_monty_p;
       std::shared_ptr<const Montgomery_Params> m_monty_q;
+      Montgomery_Int m_c_monty;
       size_t m_p_bits;
       size_t m_q_bits;
 };
@@ -288,12 +295,12 @@ RSA_PrivateKey::RSA_PrivateKey(
 
    if(d.is_zero()) {
       const BigInt phi_n = lcm(p_minus_1, q_minus_1);
-      d = inverse_mod(e, phi_n);
+      d = compute_rsa_secret_exponent(e, phi_n, p, q);
    }
 
    BigInt d1 = ct_modulo(d, p_minus_1);
    BigInt d2 = ct_modulo(d, q_minus_1);
-   BigInt c = inverse_mod(q, p);
+   BigInt c = inverse_mod_secret_prime(ct_modulo(q, p), p);
 
    RSA_PublicKey::init(std::move(n), std::move(e));
 
@@ -348,10 +355,10 @@ RSA_PrivateKey::RSA_PrivateKey(RandomNumberGenerator& rng, size_t bits, size_t e
    // This is guaranteed because p,q == 3 mod 4
    BOTAN_DEBUG_ASSERT(low_zero_bits(phi_n) == 1);
 
-   BigInt d = inverse_mod(e, phi_n);
+   BigInt d = compute_rsa_secret_exponent(e, phi_n, p, q);
    BigInt d1 = ct_modulo(d, p_minus_1);
    BigInt d2 = ct_modulo(d, q_minus_1);
-   BigInt c = inverse_mod(q, p);
+   BigInt c = inverse_mod_secret_prime(ct_modulo(q, p), p);
 
    RSA_PublicKey::init(std::move(n), std::move(e));
 
@@ -406,7 +413,7 @@ bool RSA_PrivateKey::check_key(RandomNumberGenerator& rng, bool strong) const {
    if(get_d2() != ct_modulo(get_d(), get_q() - 1)) {
       return false;
    }
-   if(get_c() != inverse_mod(get_q(), get_p())) {
+   if(get_c() != inverse_mod_secret_prime(ct_modulo(get_q(), get_p()), get_p())) {
       return false;
    }
 
@@ -424,7 +431,13 @@ bool RSA_PrivateKey::check_key(RandomNumberGenerator& rng, bool strong) const {
          return false;
       }
 
-      return KeyPair::signature_consistency_check(rng, *this, "EMSA4(SHA-256)");
+#if defined(BOTAN_HAS_PSS) && defined(BOTAN_HAS_SHA_256)
+      const std::string padding = "PSS(SHA-256)";
+#else
+      const std::string padding = "Raw";
+#endif
+
+      return KeyPair::signature_consistency_check(rng, *this, padding);
    }
 
    return true;
@@ -445,10 +458,10 @@ class RSA_Private_Operation {
             m_public(rsa.public_data()),
             m_private(rsa.private_data()),
             m_blinder(
-               m_public->get_n(),
+               m_public->reducer_mod_n(),
                rng,
                [this](const BigInt& k) { return m_public->public_op(k); },
-               [this](const BigInt& k) { return inverse_mod(k, m_public->get_n()); }),
+               [this](const BigInt& k) { return inverse_mod_rsa_public_modulus(k, m_public->get_n()); }),
             m_blinding_bits(64),
             m_max_d1_bits(m_private->p_bits() + m_blinding_bits),
             m_max_d2_bits(m_private->q_bits() + m_blinding_bits) {}
@@ -470,19 +483,15 @@ class RSA_Private_Operation {
          recovered.serialize_to(out);
       }
 
-      secure_vector<uint8_t> raw_op(const uint8_t input[], size_t input_len) {
-         secure_vector<uint8_t> out(m_public->public_modulus_bytes());
-         raw_op(out, {input, input_len});
-         return out;
-      }
-
    private:
       BigInt rsa_private_op(const BigInt& m) const {
          /*
-         TODO
-         Consider using Montgomery reduction instead of Barrett, using
-         the "Smooth RSA-CRT" method. https://eprint.iacr.org/2007/039.pdf
+         All normal implementations generate p/q of the same bitlength,
+         so this should rarely occur in practice
          */
+         if(m_private->primes_imbalanced()) {
+            return monty_exp(m_public->monty_n(), m, m_private->get_d(), m_public->get_n().bits()).value();
+         }
 
          static constexpr size_t powm_window = 4;
 
@@ -496,7 +505,7 @@ class RSA_Private_Operation {
 #if defined(BOTAN_RSA_USE_ASYNC)
          /*
          * Precompute m.sig_words in the main thread before calling async. Otherwise
-         * the two threads race (during Modular_Reducer::reduce) and while the output
+         * the two threads race (during Barrett_Reduction::reduce) and while the output
          * is correct in both threads, helgrind warns.
          */
          m.sig_words();
@@ -504,8 +513,8 @@ class RSA_Private_Operation {
          auto future_j1 = Thread_Pool::global_instance().run([this, &m, &d1_mask]() {
 #endif
             const BigInt masked_d1 = m_private->get_d1() + (d1_mask * (m_private->get_p() - 1));
-            auto powm_d1_p = monty_precompute(m_private->monty_p(), m_private->mod_p().reduce(m), powm_window);
-            BigInt j1 = monty_execute(*powm_d1_p, masked_d1, m_max_d1_bits);
+            auto powm_d1_p = monty_precompute(Montgomery_Int::from_wide_int(m_private->monty_p(), m), powm_window);
+            auto j1 = monty_execute(*powm_d1_p, masked_d1, m_max_d1_bits);
 
 #if defined(BOTAN_RSA_USE_ASYNC)
             return j1;
@@ -514,11 +523,11 @@ class RSA_Private_Operation {
 
          const BigInt d2_mask(m_blinder.rng(), m_blinding_bits);
          const BigInt masked_d2 = m_private->get_d2() + (d2_mask * (m_private->get_q() - 1));
-         auto powm_d2_q = monty_precompute(m_private->monty_q(), m_private->mod_q().reduce(m), powm_window);
-         const BigInt j2 = monty_execute(*powm_d2_q, masked_d2, m_max_d2_bits);
+         auto powm_d2_q = monty_precompute(Montgomery_Int::from_wide_int(m_private->monty_q(), m), powm_window);
+         const auto j2 = monty_execute(*powm_d2_q, masked_d2, m_max_d2_bits).value();
 
 #if defined(BOTAN_RSA_USE_ASYNC)
-         BigInt j1 = future_j1.get();
+         auto j1 = future_j1.get();
 #endif
 
          /*
@@ -527,16 +536,17 @@ class RSA_Private_Operation {
          * c = q^-1 mod p (this is precomputed)
          * h = c*(j1-j2) mod p
          * m = j2 + h*q
-         *
-         * We must avoid leaking if j1 >= j2 or not, as doing so allows deriving
-         * information about the secret prime. Do this by first adding p to j1,
-         * which should ensure the subtraction of j2 does not underflow. But
-         * this may still underflow if p and q are imbalanced in size.
          */
 
-         j1 =
-            m_private->mod_p().multiply(m_private->mod_p().reduce((m_private->get_p() + j1) - j2), m_private->get_c());
-         return j1 * m_private->get_q() + j2;
+         const auto j2_p = Montgomery_Int::from_wide_int(m_private->monty_p(), j2);
+
+         /**
+         * This doesn't quite match up with the "Smooth-CRT" proposal; there we
+         * would multiply by c * R2 so would have the effect of both multiplying
+         * by c and immediately converting from Montgomery to standard form.
+         */
+         j1 = (j1 - j2_p) * m_private->get_c_monty();
+         return j1.value() * m_private->get_q() + j2;
       }
 
       std::shared_ptr<const RSA_Public_Data> m_public;
@@ -552,13 +562,16 @@ class RSA_Private_Operation {
 class RSA_Signature_Operation final : public PK_Ops::Signature,
                                       private RSA_Private_Operation {
    public:
-      void update(const uint8_t msg[], size_t msg_len) override { m_emsa->update(msg, msg_len); }
+      void update(std::span<const uint8_t> msg) override { m_emsa->update(msg.data(), msg.size()); }
 
-      secure_vector<uint8_t> sign(RandomNumberGenerator& rng) override {
+      std::vector<uint8_t> sign(RandomNumberGenerator& rng) override {
          const size_t max_input_bits = public_modulus_bits() - 1;
          const auto msg = m_emsa->raw_data();
          const auto padded = m_emsa->encoding_of(msg, max_input_bits, rng);
-         return raw_op(padded.data(), padded.size());
+
+         std::vector<uint8_t> out(public_modulus_bytes());
+         raw_op(out, padded);
+         return out;
       }
 
       size_t signature_length() const override { return public_modulus_bytes(); }
@@ -583,12 +596,12 @@ AlgorithmIdentifier RSA_Signature_Operation::algorithm_identifier() const {
       return AlgorithmIdentifier(oid, AlgorithmIdentifier::USE_EMPTY_PARAM);
    } catch(Lookup_Error&) {}
 
-   if(emsa_name.starts_with("EMSA4(")) {
+   if(emsa_name.starts_with("PSS(")) {
       auto parameters = PSS_Params::from_emsa_name(m_emsa->name()).serialize();
-      return AlgorithmIdentifier("RSA/EMSA4", parameters);
+      return AlgorithmIdentifier("RSA/PSS", parameters);
    }
 
-   throw Not_Implemented("No algorithm identifier defined for RSA with " + emsa_name);
+   throw Invalid_Argument(fmt("Signatures using RSA/{} are not supported", emsa_name));
 }
 
 class RSA_Decryption_Operation final : public PK_Ops::Decryption_with_EME,
@@ -599,8 +612,10 @@ class RSA_Decryption_Operation final : public PK_Ops::Decryption_with_EME,
 
       size_t plaintext_length(size_t /*ctext_len*/) const override { return public_modulus_bytes(); }
 
-      secure_vector<uint8_t> raw_decrypt(const uint8_t input[], size_t input_len) override {
-         return raw_op(input, input_len);
+      secure_vector<uint8_t> raw_decrypt(std::span<const uint8_t> input) override {
+         secure_vector<uint8_t> out(public_modulus_bytes());
+         raw_op(out, input);
+         return out;
       }
 };
 
@@ -655,22 +670,20 @@ class RSA_Encryption_Operation final : public PK_Ops::Encryption_with_EME,
 
       size_t max_ptext_input_bits() const override { return public_modulus_bits() - 1; }
 
-      secure_vector<uint8_t> raw_encrypt(const uint8_t input[],
-                                         size_t input_len,
-                                         RandomNumberGenerator& /*rng*/) override {
-         BigInt input_bn(input, input_len);
-         return public_op(input_bn).serialize<secure_vector<uint8_t>>(public_modulus_bytes());
+      std::vector<uint8_t> raw_encrypt(std::span<const uint8_t> input, RandomNumberGenerator& /*rng*/) override {
+         BigInt input_bn(input);
+         return public_op(input_bn).serialize(public_modulus_bytes());
       }
 };
 
 class RSA_Verify_Operation final : public PK_Ops::Verification,
                                    private RSA_Public_Operation {
    public:
-      void update(const uint8_t msg[], size_t msg_len) override { m_emsa->update(msg, msg_len); }
+      void update(std::span<const uint8_t> msg) override { m_emsa->update(msg.data(), msg.size()); }
 
-      bool is_valid_signature(const uint8_t sig[], size_t sig_len) override {
+      bool is_valid_signature(std::span<const uint8_t> sig) override {
          const auto msg = m_emsa->raw_data();
-         const auto message_repr = recover_message_repr(sig, sig_len);
+         const auto message_repr = recover_message_repr(sig.data(), sig.size());
          return m_emsa->verify(message_repr, msg, public_modulus_bits() - 1);
       }
 
@@ -752,7 +765,7 @@ std::string parse_rsa_signature_algorithm(const AlgorithmIdentifier& alg_id) {
 
    std::string padding = sig_info[1];
 
-   if(padding == "EMSA4") {
+   if(padding == "PSS") {
       // "MUST contain RSASSA-PSS-params"
       if(alg_id.parameters().empty()) {
          throw Decoding_Error("PSS params must be provided");
@@ -761,9 +774,11 @@ std::string parse_rsa_signature_algorithm(const AlgorithmIdentifier& alg_id) {
       PSS_Params pss_params(alg_id.parameters());
 
       // hash_algo must be SHA1, SHA2-224, SHA2-256, SHA2-384 or SHA2-512
+      // We also support SHA-3 (is also supported by e.g. OpenSSL and bouncycastle)
       const std::string hash_algo = pss_params.hash_function();
       if(hash_algo != "SHA-1" && hash_algo != "SHA-224" && hash_algo != "SHA-256" && hash_algo != "SHA-384" &&
-         hash_algo != "SHA-512") {
+         hash_algo != "SHA-512" && hash_algo != "SHA-3(224)" && hash_algo != "SHA-3(256)" &&
+         hash_algo != "SHA-3(384)" && hash_algo != "SHA-3(512)") {
          throw Decoding_Error("Unacceptable hash for PSS signatures");
       }
 
@@ -773,8 +788,6 @@ std::string parse_rsa_signature_algorithm(const AlgorithmIdentifier& alg_id) {
 
       // For MGF1, it is strongly RECOMMENDED that the underlying hash
       // function be the same as the one identified by hashAlgorithm
-      //
-      // Must be SHA1, SHA2-224, SHA2-256, SHA2-384 or SHA2-512
       if(pss_params.hash_algid() != pss_params.mgf_hash_algid()) {
          throw Decoding_Error("Unacceptable MGF hash for PSS signatures");
       }
