@@ -12,9 +12,10 @@
 
 #include <botan/concepts.h>
 #include <botan/mem_ops.h>
+#include <botan/strong_type.h>
 #include <botan/types.h>
 #include <botan/internal/bswap.h>
-#include <vector>
+#include <bit>
 
 /**
  * @file loadstor.h
@@ -55,6 +56,9 @@
 
 namespace Botan {
 
+static_assert(std::endian::native == std::endian::big || std::endian::native == std::endian::little,
+              "Mixed endian systems are not supported");
+
 /**
 * Byte extraction
 * @param byte_num which byte to extract, 0 == highest byte
@@ -123,70 +127,99 @@ inline constexpr uint64_t make_uint64(
 
 namespace detail {
 
-enum class Endianness : bool {
-   Big,
-   Little,
+/**
+ * @returns the opposite endianness of the specified endianness
+ *
+ * Note this assumes that there are only two endian orderings; we
+ * do not supported mixed endian systems
+ */
+consteval std::endian opposite(std::endian endianness) {
+   if(endianness == std::endian::big) {
+      return std::endian::little;
+   } else {
+      // We already verified via static assert earlier in this file that we are
+      // running on either a big endian or little endian system
+      return std::endian::big;
+   }
+}
+
+/**
+ * Models a custom type that provides factory methods to be loaded in big- or
+ * little-endian byte order.
+ */
+template <typename T>
+concept custom_loadable = requires(std::span<const uint8_t, sizeof(T)> data) {
+   { T::load_be(data) } -> std::same_as<T>;
+   { T::load_le(data) } -> std::same_as<T>;
 };
 
 /**
- * @warning This function may return false if the native endianness is unknown
- * @returns true iff the native endianness matches the given endianness
+ * Models a custom type that provides store methods to be stored in big- or
+ * little-endian byte order.
  */
-constexpr bool is_native(Endianness endianness) {
-#if defined(BOTAN_TARGET_CPU_IS_BIG_ENDIAN)
-   return endianness == Endianness::Big;
-#elif defined(BOTAN_TARGET_CPU_IS_LITTLE_ENDIAN)
-   return endianness == Endianness::Little;
-#else
-   return false;
-#endif
-}
-
-/**
- * @warning This function may return false if the native endianness is unknown
- * @returns true iff the native endianness does not match the given endianness
- */
-constexpr bool is_opposite(Endianness endianness) {
-#if defined(BOTAN_TARGET_CPU_IS_BIG_ENDIAN)
-   return endianness == Endianness::Little;
-#elif defined(BOTAN_TARGET_CPU_IS_LITTLE_ENDIAN)
-   return endianness == Endianness::Big;
-#else
-   return false;
-#endif
-}
-
-template <Endianness endianness>
-constexpr bool native_endianness_is_unknown() {
-#if defined(BOTAN_TARGET_CPU_IS_BIG_ENDIAN) || defined(BOTAN_TARGET_CPU_IS_LITTLE_ENDIAN)
-   return false;
-#else
-   return true;
-#endif
-}
+template <typename T>
+concept custom_storable = requires(std::span<uint8_t, sizeof(T)> data, const T value) {
+   { value.store_be(data) };
+   { value.store_le(data) };
+};
 
 /**
  * Models a type that can be loaded/stored from/to a byte range.
  */
 template <typename T>
-concept unsigned_integralish = std::unsigned_integral<T> || concepts::unsigned_integral_strong_type<T> ||
-                               (std::is_enum_v<T> && std::unsigned_integral<std::underlying_type_t<T>>);
+concept unsigned_integralish =
+   std::unsigned_integral<strong_type_wrapped_type<T>> ||
+   (std::is_enum_v<T> && std::unsigned_integral<std::underlying_type_t<T>>) ||
+   (custom_loadable<strong_type_wrapped_type<T>> || custom_storable<strong_type_wrapped_type<T>>);
+
+template <typename T>
+struct wrapped_type_helper_with_enum {
+      using type = strong_type_wrapped_type<T>;
+};
+
+template <typename T>
+   requires std::is_enum_v<T>
+struct wrapped_type_helper_with_enum<T> {
+      using type = std::underlying_type_t<T>;
+};
+
+template <unsigned_integralish T>
+using wrapped_type = typename wrapped_type_helper_with_enum<T>::type;
+
+template <unsigned_integralish InT>
+constexpr auto unwrap_strong_type_or_enum(InT t) {
+   if constexpr(std::is_enum_v<InT>) {
+      // TODO: C++23: use std::to_underlying(in) instead
+      return static_cast<std::underlying_type_t<InT>>(t);
+   } else {
+      return Botan::unwrap_strong_type(t);
+   }
+}
+
+template <unsigned_integralish OutT, std::unsigned_integral T>
+constexpr auto wrap_strong_type_or_enum(T t) {
+   if constexpr(std::is_enum_v<OutT>) {
+      return static_cast<OutT>(t);
+   } else {
+      return Botan::wrap_strong_type<OutT>(t);
+   }
+}
 
 /**
  * Manually load a word from a range in either big or little endian byte order.
- * This will be used only if the endianness of the target platform is unknown at
- * compile time.
+ *
+ * This is only used at compile time.
  */
-template <Endianness endianness, std::unsigned_integral OutT, ranges::contiguous_range<uint8_t> InR>
+template <std::endian endianness, std::unsigned_integral OutT, ranges::contiguous_range<uint8_t> InR>
 inline constexpr OutT fallback_load_any(InR&& in_range) {
    std::span in{in_range};
    // clang-format off
-   if constexpr(endianness == Endianness::Big) {
+   if constexpr(endianness == std::endian::big) {
       return [&]<size_t... i>(std::index_sequence<i...>) {
          return static_cast<OutT>(((static_cast<OutT>(in[i]) << ((sizeof(OutT) - i - 1) * 8)) | ...));
       } (std::make_index_sequence<sizeof(OutT)>());
    } else {
-      static_assert(endianness == Endianness::Little);
+      static_assert(endianness == std::endian::little);
       return [&]<size_t... i>(std::index_sequence<i...>) {
          return static_cast<OutT>(((static_cast<OutT>(in[i]) << (i * 8)) | ...));
       } (std::make_index_sequence<sizeof(OutT)>());
@@ -196,19 +229,19 @@ inline constexpr OutT fallback_load_any(InR&& in_range) {
 
 /**
  * Manually store a word into a range in either big or little endian byte order.
- * This will be used only if the endianness of the target platform is unknown at
- * compile time.
+ *
+ * This will be used only at compile time.
  */
-template <Endianness endianness, std::unsigned_integral InT, ranges::contiguous_output_range<uint8_t> OutR>
+template <std::endian endianness, std::unsigned_integral InT, ranges::contiguous_output_range<uint8_t> OutR>
 inline constexpr void fallback_store_any(InT in, OutR&& out_range) {
    std::span out{out_range};
    // clang-format off
-   if constexpr(endianness == Endianness::Big) {
+   if constexpr(endianness == std::endian::big) {
       [&]<size_t... i>(std::index_sequence<i...>) {
          ((out[i] = get_byte<i>(in)), ...);
       } (std::make_index_sequence<sizeof(InT)>());
    } else {
-      static_assert(endianness == Endianness::Little);
+      static_assert(endianness == std::endian::little);
       [&]<size_t... i>(std::index_sequence<i...>) {
          ((out[i] = get_byte<sizeof(InT) - i - 1>(in)), ...);
       } (std::make_index_sequence<sizeof(InT)>());
@@ -224,7 +257,7 @@ inline constexpr void fallback_store_any(InT in, OutR&& out_range) {
  *
  * Template arguments of all overloads of load_any() share the same semantics:
  *
- *   1.  Endianness      Either `Endianness::Big` or `Endianness::Little`, that
+ *   1.  std::endian     Either `std::endian::big` or `std::endian::little`, that
  *                       will eventually select the byte order translation mode
  *                       implemented in this base function.
  *
@@ -240,56 +273,60 @@ inline constexpr void fallback_store_any(InT in, OutR&& out_range) {
  * @param in_range a fixed-length byte range
  * @return T loaded from @p in_range, as a big-endian value
  */
-template <Endianness endianness, std::unsigned_integral OutT, ranges::contiguous_range<uint8_t> InR>
-inline constexpr OutT load_any(InR&& in_range) {
+template <std::endian endianness, unsigned_integralish WrappedOutT, ranges::contiguous_range<uint8_t> InR>
+   requires(!custom_loadable<strong_type_wrapped_type<WrappedOutT>>)
+inline constexpr WrappedOutT load_any(InR&& in_range) {
+   using OutT = detail::wrapped_type<WrappedOutT>;
    ranges::assert_exact_byte_length<sizeof(OutT)>(in_range);
-   std::span in{in_range};
 
-   // At compile time we cannot use `typecast_copy` as it uses `std::memcpy`
-   // internally to copy ranges on a byte-by-byte basis, which is not allowed
-   // in a `constexpr` context.
-   if(std::is_constant_evaluated()) /* TODO: C++23: if consteval {} */ {
-      return fallback_load_any<endianness, OutT>(std::forward<InR>(in_range));
-   } else {
-      if constexpr(sizeof(OutT) == 1) {
-         return static_cast<OutT>(in[0]);
-      } else if constexpr(is_native(endianness)) {
-         return typecast_copy<OutT>(in);
-      } else if constexpr(is_opposite(endianness)) {
-         return reverse_bytes(typecast_copy<OutT>(in));
-      } else {
-         static_assert(native_endianness_is_unknown<endianness>());
+   return detail::wrap_strong_type_or_enum<WrappedOutT>([&]() -> OutT {
+      // At compile time we cannot use `typecast_copy` as it uses `std::memcpy`
+      // internally to copy ranges on a byte-by-byte basis, which is not allowed
+      // in a `constexpr` context.
+      if(std::is_constant_evaluated()) /* TODO: C++23: if consteval {} */ {
          return fallback_load_any<endianness, OutT>(std::forward<InR>(in_range));
+      } else {
+         std::span in{in_range};
+         if constexpr(sizeof(OutT) == 1) {
+            return static_cast<OutT>(in[0]);
+         } else if constexpr(endianness == std::endian::native) {
+            return typecast_copy<OutT>(in);
+         } else {
+            static_assert(opposite(endianness) == std::endian::native);
+            return reverse_bytes(typecast_copy<OutT>(in));
+         }
       }
+   }());
+}
+
+/**
+ * Load a custom object from a range in either big or little endian byte order
+ *
+ * This is the base implementation for custom objects (e.g. SIMD type wrappres),
+ * all other overloads are just convenience overloads.
+ *
+ * @param in_range a fixed-length byte range
+ * @return T loaded from @p in_range, as a big-endian value
+ */
+template <std::endian endianness, unsigned_integralish WrappedOutT, ranges::contiguous_range<uint8_t> InR>
+   requires(custom_loadable<strong_type_wrapped_type<WrappedOutT>>)
+inline constexpr WrappedOutT load_any(InR&& in_range) {
+   using OutT = detail::wrapped_type<WrappedOutT>;
+   ranges::assert_exact_byte_length<sizeof(OutT)>(in_range);
+   std::span<const uint8_t, sizeof(OutT)> ins{in_range};
+   if constexpr(endianness == std::endian::big) {
+      return wrap_strong_type<WrappedOutT>(OutT::load_be(ins));
+   } else {
+      return wrap_strong_type<WrappedOutT>(OutT::load_le(ins));
    }
 }
 
-/**
- * Overload for loading into a strong type holding an unsigned integer
- */
-template <Endianness endianness, concepts::unsigned_integral_strong_type OutT, ranges::contiguous_range<uint8_t> InR>
-inline constexpr OutT load_any(InR&& in_range) {
-   using underlying_type = typename OutT::wrapped_type;
-   return OutT{load_any<endianness, underlying_type>(std::forward<InR>(in_range))};
-}
-
-/**
- * Overload for loading into an enum type that uses an unsigned integer as its
- * underlying type.
- */
-template <Endianness endianness, typename OutT, ranges::contiguous_range<uint8_t> InR>
-   requires(std::is_enum_v<OutT> && std::unsigned_integral<std::underlying_type_t<OutT>>)
-inline constexpr OutT load_any(InR&& in_range) {
-   using underlying_type = std::underlying_type_t<OutT>;
-   return static_cast<OutT>(load_any<endianness, underlying_type>(std::forward<InR>(in_range)));
-}
-
 /**
  * Load many unsigned integers
  * @param in   a fixed-length span to some bytes
  * @param outs a arbitrary-length parameter list of unsigned integers to be loaded
  */
-template <Endianness endianness, typename OutT, ranges::contiguous_range<uint8_t> InR, unsigned_integralish... Ts>
+template <std::endian endianness, typename OutT, ranges::contiguous_range<uint8_t> InR, unsigned_integralish... Ts>
    requires(sizeof...(Ts) > 0) && ((std::same_as<AutoDetect, OutT> && all_same_v<Ts...>) ||
                                    (unsigned_integralish<OutT> && all_same_v<OutT, Ts...>))
 inline constexpr void load_any(InR&& in, Ts&... outs) {
@@ -309,7 +346,7 @@ inline constexpr void load_any(InR&& in, Ts&... outs) {
  * @param out the output range of words
  * @param in the input range of bytes
  */
-template <Endianness endianness,
+template <std::endian endianness,
           typename OutT,
           ranges::contiguous_output_range OutR,
           ranges::contiguous_range<uint8_t> InR>
@@ -317,9 +354,9 @@ template <Endianness endianness,
             (std::same_as<AutoDetect, OutT> || std::same_as<OutT, std::ranges::range_value_t<OutR>>))
 inline constexpr void load_any(OutR&& out, InR&& in) {
    ranges::assert_equal_byte_lengths(out, in);
+   using element_type = std::ranges::range_value_t<OutR>;
 
    auto load_elementwise = [&] {
-      using element_type = std::ranges::range_value_t<OutR>;
       constexpr size_t bytes_per_element = sizeof(element_type);
       std::span<const uint8_t> in_s(in);
       for(auto& out_elem : out) {
@@ -334,7 +371,7 @@ inline constexpr void load_any(OutR&& out, InR&& in) {
    if(std::is_constant_evaluated()) /* TODO: C++23: if consteval {} */ {
       load_elementwise();
    } else {
-      if constexpr(is_native(endianness)) {
+      if constexpr(endianness == std::endian::native && !custom_loadable<element_type>) {
          typecast_copy(out, in);
       } else {
          load_elementwise();
@@ -353,7 +390,7 @@ inline constexpr void load_any(OutR&& out, InR&& in) {
  * @param in_range a statically-sized range with some bytes
  * @return T loaded from in
  */
-template <Endianness endianness, typename OutT, ranges::contiguous_range<uint8_t> InR>
+template <std::endian endianness, typename OutT, ranges::contiguous_range<uint8_t> InR>
    requires(std::same_as<AutoDetect, OutT> ||
             ((ranges::statically_spanable_range<OutT> || concepts::resizable_container<OutT>) &&
              unsigned_integralish<typename OutT::value_type>))
@@ -413,7 +450,7 @@ inline constexpr auto load_any(InR&& in_range) {
  * @param off an offset into the array
  * @return off'th T of in, as a big-endian value
  */
-template <Endianness endianness, unsigned_integralish OutT>
+template <std::endian endianness, unsigned_integralish OutT>
 inline constexpr OutT load_any(const uint8_t in[], size_t off) {
    // asserts that *in points to enough bytes to read at offset off
    constexpr size_t out_size = sizeof(OutT);
@@ -425,7 +462,7 @@ inline constexpr OutT load_any(const uint8_t in[], size_t off) {
  * @param in   a pointer to some bytes
  * @param outs a arbitrary-length parameter list of unsigned integers to be loaded
  */
-template <Endianness endianness, typename OutT, unsigned_integralish... Ts>
+template <std::endian endianness, typename OutT, unsigned_integralish... Ts>
    requires(sizeof...(Ts) > 0 && all_same_v<Ts...> &&
             ((std::same_as<AutoDetect, OutT> && all_same_v<Ts...>) ||
              (unsigned_integralish<OutT> && all_same_v<OutT, Ts...>)))
@@ -441,7 +478,7 @@ inline constexpr void load_any(const uint8_t in[], Ts&... outs) {
  * @param in the input array of bytes
  * @param count how many words are in in
  */
-template <Endianness endianness, typename OutT, unsigned_integralish T>
+template <std::endian endianness, typename OutT, unsigned_integralish T>
    requires(std::same_as<AutoDetect, OutT> || std::same_as<T, OutT>)
 inline constexpr void load_any(T out[], const uint8_t in[], size_t count) {
    // asserts that *in and *out point to the correct amount of memory
@@ -456,7 +493,7 @@ inline constexpr void load_any(T out[], const uint8_t in[], size_t count) {
  */
 template <typename OutT = detail::AutoDetect, typename... ParamTs>
 inline constexpr auto load_le(ParamTs&&... params) {
-   return detail::load_any<detail::Endianness::Little, OutT>(std::forward<ParamTs>(params)...);
+   return detail::load_any<std::endian::little, OutT>(std::forward<ParamTs>(params)...);
 }
 
 /**
@@ -465,7 +502,7 @@ inline constexpr auto load_le(ParamTs&&... params) {
  */
 template <typename OutT = detail::AutoDetect, typename... ParamTs>
 inline constexpr auto load_be(ParamTs&&... params) {
-   return detail::load_any<detail::Endianness::Big, OutT>(std::forward<ParamTs>(params)...);
+   return detail::load_any<std::endian::big, OutT>(std::forward<ParamTs>(params)...);
 }
 
 namespace detail {
@@ -480,12 +517,15 @@ namespace detail {
  * as those of load_any(). See the documentation of this function for more
  * details.
  *
- * @param in an unsigned integral to be stored
- * @param out_range a byte range to store the word into
+ * @param wrapped_in an unsigned integral to be stored
+ * @param out_range  a byte range to store the word into
  */
-template <Endianness endianness, std::unsigned_integral InT, ranges::contiguous_output_range<uint8_t> OutR>
-inline constexpr void store_any(InT in, OutR&& out_range) {
-   ranges::assert_exact_byte_length<sizeof(InT)>(out_range);
+template <std::endian endianness, unsigned_integralish WrappedInT, ranges::contiguous_output_range<uint8_t> OutR>
+   requires(!custom_storable<strong_type_wrapped_type<WrappedInT>>)
+inline constexpr void store_any(WrappedInT wrapped_in, OutR&& out_range) {
+   const auto in = detail::unwrap_strong_type_or_enum(wrapped_in);
+   using InT = decltype(in);
+   ranges::assert_exact_byte_length<sizeof(in)>(out_range);
    std::span out{out_range};
 
    // At compile time we cannot use `typecast_copy` as it uses `std::memcpy`
@@ -496,38 +536,36 @@ inline constexpr void store_any(InT in, OutR&& out_range) {
    } else {
       if constexpr(sizeof(InT) == 1) {
          out[0] = static_cast<uint8_t>(in);
-      } else if constexpr(is_native(endianness)) {
+      } else if constexpr(endianness == std::endian::native) {
          typecast_copy(out, in);
-      } else if constexpr(is_opposite(endianness)) {
-         typecast_copy(out, reverse_bytes(in));
       } else {
-         static_assert(native_endianness_is_unknown<endianness>());
-         return fallback_store_any<endianness, InT>(in, std::forward<OutR>(out_range));
+         static_assert(opposite(endianness) == std::endian::native);
+         typecast_copy(out, reverse_bytes(in));
       }
    }
 }
 
 /**
- * Overload for loading into a strong type holding an unsigned integer
+ * Store a custom word in either big or little endian byte order into a range
+ *
+ * This is the base implementation for storing custom objects, all other
+ * overloads are just convenience overloads.
+ *
+ * @param wrapped_in a custom object to be stored
+ * @param out_range  a byte range to store the word into
  */
-template <Endianness endianness,
-          concepts::unsigned_integral_strong_type InT,
-          ranges::contiguous_output_range<uint8_t> OutR>
-inline constexpr void store_any(InT in, OutR&& out_range) {
-   using underlying_type = typename InT::wrapped_type;
-   store_any<endianness, underlying_type>(in.get(), std::forward<OutR>(out_range));
-}
-
-/**
- * Overload for storing an enum type that uses an unsigned integer as its
- * underlying type.
- */
-template <Endianness endianness, typename InT, ranges::contiguous_output_range<uint8_t> OutR>
-   requires(std::is_enum_v<InT> && std::unsigned_integral<std::underlying_type_t<InT>>)
-inline constexpr void store_any(InT in, OutR&& out_range) {
-   using underlying_type = std::underlying_type_t<InT>;
-   // TODO: C++23: use std::to_underlying(in) instead
-   store_any<endianness, underlying_type>(static_cast<underlying_type>(in), std::forward<OutR>(out_range));
+template <std::endian endianness, unsigned_integralish WrappedInT, ranges::contiguous_output_range<uint8_t> OutR>
+   requires(custom_storable<strong_type_wrapped_type<WrappedInT>>)
+inline constexpr void store_any(WrappedInT wrapped_in, OutR&& out_range) {
+   const auto in = detail::unwrap_strong_type_or_enum(wrapped_in);
+   using InT = decltype(in);
+   ranges::assert_exact_byte_length<sizeof(in)>(out_range);
+   std::span<uint8_t, sizeof(InT)> outs{out_range};
+   if constexpr(endianness == std::endian::big) {
+      in.store_be(outs);
+   } else {
+      in.store_le(outs);
+   }
 }
 
 /**
@@ -535,7 +573,7 @@ inline constexpr void store_any(InT in, OutR&& out_range) {
  * @param out a sized range of some bytes
  * @param ins a arbitrary-length parameter list of unsigned integers to be stored
  */
-template <Endianness endianness,
+template <std::endian endianness,
           typename InT,
           ranges::contiguous_output_range<uint8_t> OutR,
           unsigned_integralish... Ts>
@@ -557,16 +595,16 @@ inline constexpr void store_any(OutR&& out, Ts... ins) {
  * @param out the output range of bytes
  * @param in the input range of words
  */
-template <Endianness endianness,
+template <std::endian endianness,
           typename InT,
           ranges::contiguous_output_range<uint8_t> OutR,
           ranges::spanable_range InR>
    requires(std::same_as<AutoDetect, InT> || std::same_as<InT, std::ranges::range_value_t<InR>>)
 inline constexpr void store_any(OutR&& out, InR&& in) {
    ranges::assert_equal_byte_lengths(out, in);
+   using element_type = std::ranges::range_value_t<InR>;
 
    auto store_elementwise = [&] {
-      using element_type = std::ranges::range_value_t<InR>;
       constexpr size_t bytes_per_element = sizeof(element_type);
       std::span<uint8_t> out_s(out);
       for(auto in_elem : in) {
@@ -581,7 +619,7 @@ inline constexpr void store_any(OutR&& out, InR&& in) {
    if(std::is_constant_evaluated()) /* TODO: C++23: if consteval {} */ {
       store_elementwise();
    } else {
-      if constexpr(is_native(endianness)) {
+      if constexpr(endianness == std::endian::native && !custom_storable<element_type>) {
          typecast_copy(out, in);
       } else {
          store_elementwise();
@@ -603,7 +641,7 @@ inline constexpr void store_any(OutR&& out, InR&& in) {
  * @param in an unsigned integer to be stored
  * @param out_range a range of bytes to store the word into
  */
-template <Endianness endianness, typename InT, unsigned_integralish T, ranges::contiguous_output_range<uint8_t> OutR>
+template <std::endian endianness, typename InT, unsigned_integralish T, ranges::contiguous_output_range<uint8_t> OutR>
    requires std::same_as<AutoDetect, InT>
 inline constexpr void store_any(T in, OutR&& out_range) {
    store_any<endianness, T>(in, std::forward<OutR>(out_range));
@@ -618,7 +656,7 @@ inline constexpr void store_any(T in, OutR&& out_range) {
  * @param in_range a range of words that should be stored
  * @return a container of bytes that contains the stored words
  */
-template <Endianness endianness, typename OutR, ranges::spanable_range InR>
+template <std::endian endianness, typename OutR, ranges::spanable_range InR>
    requires(std::same_as<AutoDetect, OutR> ||
             (ranges::statically_spanable_range<OutR> && std::default_initializable<OutR>) ||
             concepts::resizable_byte_buffer<OutR>)
@@ -653,7 +691,7 @@ inline constexpr auto store_any(InR&& in_range) {
  * @param ins some words that should be stored
  * @return a container of bytes that contains the stored words
  */
-template <Endianness endianness, typename OutR, unsigned_integralish... Ts>
+template <std::endian endianness, typename OutR, unsigned_integralish... Ts>
    requires all_same_v<Ts...>
 inline constexpr auto store_any(Ts... ins) {
    return store_any<endianness, OutR>(std::array{ins...});
@@ -668,7 +706,7 @@ inline constexpr auto store_any(Ts... ins) {
  * @param in the input unsigned integer
  * @param out the byte array to write to
  */
-template <Endianness endianness, typename InT, unsigned_integralish T>
+template <std::endian endianness, typename InT, unsigned_integralish T>
    requires(std::same_as<AutoDetect, InT> || std::same_as<T, InT>)
 inline constexpr void store_any(T in, uint8_t out[]) {
    // asserts that *out points to enough bytes to write into
@@ -680,7 +718,7 @@ inline constexpr void store_any(T in, uint8_t out[]) {
  * @param ins a arbitrary-length parameter list of unsigned integers to be stored
  * @param out the byte array to write to
  */
-template <Endianness endianness, typename InT, unsigned_integralish T0, unsigned_integralish... Ts>
+template <std::endian endianness, typename InT, unsigned_integralish T0, unsigned_integralish... Ts>
    requires(std::same_as<AutoDetect, InT> || std::same_as<T0, InT>) && all_same_v<T0, Ts...>
 inline constexpr void store_any(uint8_t out[], T0 in0, Ts... ins) {
    constexpr auto bytes = sizeof(in0) + (sizeof(ins) + ... + 0);
@@ -696,7 +734,7 @@ inline constexpr void store_any(uint8_t out[], T0 in0, Ts... ins) {
  */
 template <typename ModifierT = detail::AutoDetect, typename... ParamTs>
 inline constexpr auto store_le(ParamTs&&... params) {
-   return detail::store_any<detail::Endianness::Little, ModifierT>(std::forward<ParamTs>(params)...);
+   return detail::store_any<std::endian::little, ModifierT>(std::forward<ParamTs>(params)...);
 }
 
 /**
@@ -705,12 +743,12 @@ inline constexpr auto store_le(ParamTs&&... params) {
  */
 template <typename ModifierT = detail::AutoDetect, typename... ParamTs>
 inline constexpr auto store_be(ParamTs&&... params) {
-   return detail::store_any<detail::Endianness::Big, ModifierT>(std::forward<ParamTs>(params)...);
+   return detail::store_any<std::endian::big, ModifierT>(std::forward<ParamTs>(params)...);
 }
 
 namespace detail {
 
-template <Endianness endianness, unsigned_integralish T>
+template <std::endian endianness, unsigned_integralish T>
 inline size_t copy_out_any_word_aligned_portion(std::span<uint8_t>& out, std::span<const T>& in) {
    const size_t full_words = out.size() / sizeof(T);
    const size_t full_word_bytes = full_words * sizeof(T);
@@ -735,7 +773,7 @@ template <ranges::spanable_range InR>
 inline void copy_out_be(std::span<uint8_t> out, InR&& in) {
    using T = std::ranges::range_value_t<InR>;
    std::span<const T> in_s{in};
-   const auto remaining_bytes = detail::copy_out_any_word_aligned_portion<detail::Endianness::Big>(out, in_s);
+   const auto remaining_bytes = detail::copy_out_any_word_aligned_portion<std::endian::big>(out, in_s);
 
    // copy remaining bytes as a partial word
    for(size_t i = 0; i < remaining_bytes; ++i) {
@@ -751,7 +789,7 @@ template <ranges::spanable_range InR>
 inline void copy_out_le(std::span<uint8_t> out, InR&& in) {
    using T = std::ranges::range_value_t<InR>;
    std::span<const T> in_s{in};
-   const auto remaining_bytes = detail::copy_out_any_word_aligned_portion<detail::Endianness::Little>(out, in_s);
+   const auto remaining_bytes = detail::copy_out_any_word_aligned_portion<std::endian::little>(out, in_s);
 
    // copy remaining bytes as a partial word
    for(size_t i = 0; i < remaining_bytes; ++i) {
