@@ -254,7 +254,33 @@ class Kyber_Encoding_Test : public Text_Based_Test {
 
             result.test_eq("sk's encoding of pk", skr->public_key_bits(), pk_raw);
             result.test_eq("sk's encoding of sk", skr->private_key_bits(), sk_raw);
-            result.test_eq("pk's encoding of pk", skr->public_key_bits(), pk_raw);
+            result.test_eq("pk's encoding of pk", pkr->public_key_bits(), pk_raw);
+
+            // expanded vs seed encoding
+            if(skr->private_key_format() == Botan::MlPrivateKeyFormat::Seed) {
+               result.test_eq("sk's seed encoding of sk",
+                              skr->private_key_bits_with_format(Botan::MlPrivateKeyFormat::Seed),
+                              sk_raw);
+               const auto skr_expanded = std::make_unique<Botan::Kyber_PrivateKey>(
+                  skr->private_key_bits_with_format(Botan::MlPrivateKeyFormat::Expanded), mode);
+               result.test_eq("sk's expanded encoding consistency",
+                              skr->private_key_bits_with_format(Botan::MlPrivateKeyFormat::Expanded),
+                              skr_expanded->private_key_bits_with_format(Botan::MlPrivateKeyFormat::Expanded));
+               result.test_throws<Botan::Encoding_Error>("expect no seed in expanded sk", [&] {
+                  skr_expanded->private_key_bits_with_format(Botan::MlPrivateKeyFormat::Seed);
+               });
+
+               const auto encapsulation = Botan::PK_KEM_Encryptor(*pkr, "Raw").encrypt(rng());
+               result.test_eq(
+                  "expanded sk decapsulation",
+                  Botan::PK_KEM_Decryptor(*skr_expanded, rng(), "Raw").decrypt(encapsulation.encapsulated_shared_key()),
+                  encapsulation.shared_key());
+
+            } else {
+               result.test_eq("sk's expanded encoding of sk",
+                              skr->private_key_bits_with_format(Botan::MlPrivateKeyFormat::Expanded),
+                              sk_raw);
+            }
          }
 
          return result;
@@ -309,9 +335,8 @@ void test_compress(Test::Result& res) {
 
    for(uint16_t x = 0; x < q; ++x) {
       const uint32_t c = Kyber_Algos::compress<d>(x);
-      const auto twotothed = (uint32_t(1) << d);
-      const auto expected = (static_cast<double>(twotothed) / q) * x;
-      const auto e = static_cast<uint32_t>(std::round(expected)) % twotothed;
+      constexpr auto twotothed = (uint32_t(1) << d);
+      const auto e = ((twotothed * x + (q / 2)) / q) % twotothed;
 
       if(c != e) {
          res.test_failure(fmt("compress<{}>({}) = {}; expected {}", d, x, c, e));
@@ -330,13 +355,12 @@ void test_decompress(Test::Result& result) {
 
    result.start_timer();
 
-   const auto twotothed = (uint32_t(1) << d);
    using from_t = std::conditional_t<d <= 8, uint8_t, uint16_t>;
+   const from_t twotothed = static_cast<from_t>(from_t(1) << d);
 
    for(from_t y = 0; y < twotothed; ++y) {
       const uint32_t c = Kyber_Algos::decompress<d>(y);
-      const auto expected = (static_cast<double>(q) / twotothed) * y;
-      const auto e = static_cast<uint32_t>(std::round(expected)) % q;
+      const uint32_t e = (q * y + (twotothed / 2)) / twotothed;
 
       if(c != e) {
          result.test_failure(fmt("decompress<{}>({}) = {}; expected {}", d, static_cast<uint16_t>(y), c, e));
